# -*- coding: utf_8 -*-
from panda3d.core import *
from direct.filter.FilterManager import FilterManager

class SSR():
    
    def __init__(self, app):
        self.base = app
        self.base.cam.node().getLens().setNearFar(1,100)
        self.manager = FilterManager(self.base.win, self.base.cam)
        self.albedo = Texture()
        self.depth = Texture()
        self.normal = Texture()
        #self.g_buffer = self.make_geom_buffer(-1)
        #finalquad = self.manager.renderSceneInto(colortex = self.albedo)
        finalquad = self.manager.renderSceneInto(colortex = self.albedo, 
                                                 depthtex = self.depth, 
                                                 auxtex = self.normal,
                                                 auxbits = AuxBitplaneAttrib.ABOAuxNormal)
        ssr_filter = self.make_filter_buffer(self.base.win, 'ssr_buffer', 1, 'ssr.sha',
                                        texture = None,
                                        inputs = [("albedo", self.albedo),
                                                  ("depth", self.depth),
                                                  ("normal", self.normal),
                                                  ("reflectivity", Texture()),
                                                  ("noise", loader.loadTexture('./tex/noise.png')),
                                                  ("mcamera", self.base.cam),
                                                  ("proj_mat", self.base.cam.node().getLens().getProjectionMat()),
                                                  ("proj_mat_inv", self.base.cam.node().getLens().getProjectionMatInv())
                                                  ]
                                            )
        #blurXBuffer = self.make_filter_buffer(ssr_filter,  "Blur X", 2, 'XBlurShader.sha')
        #blurYBuffer = self.make_filter_buffer(blurXBuffer, "Blur Y", 3, 'YBlurShader.sha')
        finalquad.setShader(Shader.load("mix.sha"))
        finalquad.setShaderInput("albedo", self.albedo)
        #finalquad.setShaderInput("reflection", blurYBuffer.getTexture())
        finalquad.setShaderInput("reflection", ssr_filter.getTexture())
        
        #finalquad.setShader(Shader.load("ssr.sha"))
        #finalquad.setShaderInput("albedo", self.albedo)
        #finalquad.setShaderInput("depth", self.depth)
        #finalquad.setShaderInput("normal", self.normal)
        #finalquad.setShaderInput("reflectivity", self.reflect)
        #finalquad.setShaderInput("reflectivity", Texture())
        #finalquad.setShaderInput("noise", loader.loadTexture('./tex/noise.png'))
        #finalquad.setTexture(ssr_filter.getTexture())

        
    def make_filter_buffer(self, srcbuffer, name, sort, prog, texture = None, inputs = None):
        blurBuffer=base.win.makeTextureBuffer(name, 512, 512)
        blurBuffer.setSort(sort)
        blurBuffer.setClearColor(Vec4(1,0,0,1))
        blurCamera=base.makeCamera2d(blurBuffer)
        blurScene=NodePath("new Scene")
        blurCamera.node().setScene(blurScene)
        shader = loader.loadShader(prog)
        card = srcbuffer.getTextureCard()
        if texture:
            card.setTexture(texture)
        card.reparentTo(blurScene)
        card.setShader(shader)
        if inputs:
            for name, val in inputs:
                card.setShaderInput(name, val)
        return blurBuffer
    '''
    def make_geom_buffer(self, sort): # Not used yet
        props = FrameBufferProperties()
        props.setRgbColor(1)
        props.setDepthBits(1)
        props.setAlphaBits(1)
        props.setAuxRgba(1)
        depth = Texture()
        depth.setFormat( Texture.FDepthComponent )
        mybuffer = base.win.makeTextureBuffer('geom_buffer', 
                                              512, 512, 
                                              to_ram = False, 
                                              tex = depth, 
                                              fbp = props)
        mybuffer.setClearColor(Vec4(1,1,1,0)) 
        mybuffer.setSort(sort)
        mycamera = base.makeCamera(mybuffer)
        mycamera.node().setLens(base.cam.node().getLens())
        tempnode = NodePath(PandaNode("temp node"))
        tempnode.setShader(loader.loadShader("g_buffer.sha"))
        mycamera.node().setInitialState(tempnode.getState())
        tempnode.removeNode()
        normal = Texture()
        reflect = Texture()
        mybuffer.addRenderTexture(normal, 
                                GraphicsOutput.RTMBindOrCopy, 
                                GraphicsOutput.RTPColor)
        mybuffer.addRenderTexture(reflect, 
                                GraphicsOutput.RTMBindOrCopy, 
                                GraphicsOutput.RTPAuxRgba0)
        mybuffer.setClearActive(mybuffer.RTPAuxRgba0, True)
        self.depth = depth
        self.normal = normal
        self.reflect = reflect
        
        return mybuffer
    '''

def test_scene(scene_root):
    back = loader.loadModel('test_scene')
    back.setScale(5)
    back.reparentTo(scene_root)
    box = loader.loadModel('box')
    box.reparentTo(scene_root)
    box.setScale(1,1,6)
    tp = loader.loadModel('teapot')
    tp.reparentTo(scene_root)
    tp.setPos(-2, 2, 0)
    tp.setScale(0.5)
    sm = loader.loadModel('smiley')
    sm.reparentTo(scene_root)
    sm.setPos(0, 4, 1)
    sm.setColor(Vec4(1, 0, 0, 1))

    dlight = DirectionalLight('dlight')
    dlight.setColor(VBase4(0.5, 0.5, 0.5, 1))
    dlnp = scene_root.attachNewNode(dlight)
    dlnp.setHpr(0, -60, 0)
    scene_root.setLight(dlnp)
    alight = AmbientLight('alight')
    alight.setColor(VBase4(0.5, 0.5, 0.5, 1))
    alnp = scene_root.attachNewNode(alight)
    scene_root.setLight(alnp)

if __name__ == '__main__':
    loadPrcFileData("", "prefer-parasite-buffer #t")
    loadPrcFileData("", "show-buffers #t")
    loadPrcFileData("", "show-frame-rate-meter #t")
    #loadPrcFileData("", "dump-generated-shaders #t")
    import direct.directbase.DirectStart
    import random
    test_scene(render)
    render.setShaderAuto()
    ssr = SSR(base)
    #print base.cam.node().getLens().getProjectionMat()
    #print base.cam.node().getLens().getFilmMat()
    #print base.cam.node().getLens().getViewMat()
    
    run()
